package ru.dbotthepony.mc.otm.container.slotted

import net.minecraft.core.HolderLookup
import net.minecraft.core.registries.BuiltInRegistries
import net.minecraft.nbt.CompoundTag
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.container.IFilteredAutomatedContainerSlot
import ru.dbotthepony.mc.otm.core.nbt.set
import ru.dbotthepony.mc.otm.core.registryName
import java.util.Collections

open class FilteredContainerSlot(
	container: SlottedContainer,
	slot: Int
) : ContainerSlot(container, slot), IFilteredAutomatedContainerSlot {
	override var filter: Item? = null
		set(value) {
			if (field !== value) {
				field = value
				container.notifyChanged()
			}
		}

	override fun clear() {
		super.clear()
		filter = null
	}

	override fun serializeNBT(provider: HolderLookup.Provider): CompoundTag {
		return super.serializeNBT(provider).also {
			if (filter != null)
				it["filter"] = filter!!.registryName!!.toString()
		}
	}

	override fun deserializeNBT(provider: HolderLookup.Provider, nbt: CompoundTag) {
		super.deserializeNBT(provider, nbt)

		if ("filter" in nbt) {
			filter = BuiltInRegistries.ITEM.get(ResourceLocation.parse(nbt.getString("filter")))
		}
	}

	class Simple(
		private val listener: (new: ItemStack, old: ItemStack) -> Unit = { _, _ -> },
		private val maxStackSize: Int = Item.DEFAULT_MAX_STACK_SIZE,
		private val canAutomationPlaceItem: AutomationPlaceItem<FilteredContainerSlot> = AutomationPlaceItem { _, _ -> true },
		private val canAutomationTakeItem: AutomationTakeItem<FilteredContainerSlot> = AutomationTakeItem { _, _ -> true },
		private val modifyAutomationPlaceCount: AutomationModifyPlaceCount<FilteredContainerSlot> = AutomationModifyPlaceCount { _, item -> item.count },
		private val modifyAutomationExtractionCount: AutomationModifyExtractionCount<FilteredContainerSlot> = AutomationModifyExtractionCount { _, desired -> desired },
	) : SlottedContainer.SlotProvider<FilteredContainerSlot> {
		constructor(
			listener: (new: ItemStack, old: ItemStack) -> Unit = { _, _ -> },
			maxStackSize: Int = Item.DEFAULT_MAX_STACK_SIZE,
			filter: AutomationFilter<FilteredContainerSlot> = AutomationFilters.ALLOW
		) : this(listener, maxStackSize, filter, filter, filter, filter)

		private open inner class Instance(container: SlottedContainer, slot: Int) : FilteredContainerSlot(container, slot) {
			override val maxStackSize: Int
				get() = this@Simple.maxStackSize

			override fun notifyChanged(old: ItemStack) {
				super.notifyChanged(old)
				listener(item, old)
			}

			override fun canAutomationPlaceItem(itemStack: ItemStack): Boolean {
				return super.canAutomationPlaceItem(itemStack) && canAutomationPlaceItem.canAutomationPlaceItem(this, itemStack)
			}

			override fun canAutomationTakeItem(desired: Int): Boolean {
				return super.canAutomationTakeItem(desired) && canAutomationTakeItem.canAutomationTakeItem(this, desired)
			}

			override fun modifyAutomationPlaceCount(itemStack: ItemStack): Int {
				return modifyAutomationPlaceCount.modifyAutomationPlaceCount(this, itemStack)
			}

			override fun modifyAutomationExtractionCount(desired: Int): Int {
				return modifyAutomationExtractionCount.modifyAutomationExtractionCount(this, desired)
			}
		}

		override fun create(container: SlottedContainer, index: Int): FilteredContainerSlot {
			return Instance(container, index)
		}
	}
}
