package ru.dbotthepony.mc.otm.container.util

import net.minecraft.world.Container
import net.minecraft.world.inventory.Slot
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.container.IContainerSlot
import ru.dbotthepony.mc.otm.container.IEnhancedContainer
import ru.dbotthepony.mc.otm.container.get
import ru.dbotthepony.mc.otm.core.collect.filter
import ru.dbotthepony.mc.otm.core.collect.map
import ru.dbotthepony.mc.otm.core.isNotEmpty

fun Container.containerSlot(slot: Int): IContainerSlot {
	if (this is IEnhancedContainer<*>) {
		return containerSlot(slot)
	} else {
		return IContainerSlot.Simple(slot, this)
	}
}

/**
 * Returns [IContainerSlot] only if this container is [IEnhancedContainer]
 */
fun Container.containerSlotOrNull(slot: Int): IContainerSlot? {
	if (this is IEnhancedContainer<*>) {
		return containerSlot(slot)
	} else {
		return null
	}
}

fun Slot.containerSlot(): IContainerSlot {
	return container.containerSlot(slotIndex)
}

fun Slot.containerSlotOrNull(): IContainerSlot? {
	return container.containerSlotOrNull(slotIndex)
}

operator fun Container.iterator(): Iterator<ItemStack> {
	if (this is IEnhancedContainer<*>) {
		return iterator()
	} else {
		return (0 until containerSize).iterator().map { this[it] }.filter { it.isNotEmpty }
	}
}

fun Container.slotIterator(): Iterator<IContainerSlot> {
	if (this is IEnhancedContainer<*>) {
		return slotIterator()
	} else {
		return (0 until containerSize).iterator().map { IContainerSlot.Simple(it, this) }
	}
}

fun Container.nonEmptySlotIterator(): Iterator<IContainerSlot> {
	if (this is IEnhancedContainer<*>) {
		return nonEmptySlotIterator()
	} else {
		return (0 until containerSize).iterator().filter { this[it].isNotEmpty }.map { IContainerSlot.Simple(it, this) }
	}
}
